function [u0p,v0p,Ap,Caap,Ap_true] = generate_AC_pair(...
    Bv,Rm,X1hev,X2hev,n1,n2,scales,sigma_bb,sigma_a0,sigma_b0)
%% Generate uncertain AC pair
%
% Bv,Rm         right camera
% X1hev,X2hev   2 3D points, homogeneous
% n1,n2         2vectors of normal components n=[A1,A2]' 
% scales        scales of the extrected keypoints
% sigma_bb      stdv of bias of keypoints
% sigma_a0, sigma_b0 std of affinity
%
% u0p,v0p       2x2 matrix reference points left and right u0p(i,:) ...
% Ap            3x3x2 tensor of homogeneous affinities
% Caap          6x6x2 tensor of Covariance matrices of affine parameters
% Ct1, Ct2      3 x 9 matrices for linear constraints
%% camera geometry
P1m  = [eye(3),zeros(3,1)];
P2m  = Rm * [eye(3),Bv];
E_true = calc_S(Bv)*Rm';
E_true_t = E_true';

%% Jacobians of X wrt map
JZ1m = [eye(2);n1'];
JZ2m = [eye(2);n2'];

%% 1. AC
x01hv = P1m * X1hev;                            % left references
x01v  = x01hv(1:2,:)/x01hv(3);
z1hv  = P2m * X1hev;                            % right correspondence
z1v   = z1hv(1:2,:)/z1hv(3);
b1v   = sigma_bb*randn(2,1);                    % bias of right reference
z01v  = z1v - b1v;                               % right reference
Jc11m = 1/x01hv(3)^2 * [x01hv(3)*eye(2),-x01hv(1:2)];
Jc21m = 1/z1hv(3)^2  * [z1hv(3) *eye(2),-z1hv(1:2)];
J11m  = Jc11m * JZ1m;
J21m  = Jc21m * Rm * JZ1m;
A001m = J21m / J11m;                             % 2x2 affinity
A1m   = [A001m,b1v;0 0 1];                        % 3x3 affinity
A01m  = [A001m;zeros(1,2)];
Caa1m = diag([sigma_a0^2/scales(1)^4*[1,1,1,1],...
    sigma_b0^2/scales(1)^2*[1,1]]);

% check epipolar geometry points
cA1      = [eye(2),x01v;[0 0 1]]'*E_true *[z1v ;1] + ...
           [A001m,z1v  ;[0,0,1]]'*E_true'*[x01v;1];
cp1      = x01hv'*E_true*z1hv;
Ct1      =  kron([eye(2),x01v;[0 0 1]]',[z1v;1]')...
          +  kron([x01v;1]',[A001m,z1v;[0,0,1]]');
2*kron([x01v;1]',[z1v;1]');
c1       = Ct1*E_true_t(:);

%% 2. AC
x02hv = P1m * X2hev;                            % left references
x02v  = x02hv(1:2,:)/x02hv(3);
z2hv  = P2m * X2hev;                            % right correspondence
z2v   = z2hv(1:2,:)/z2hv(3);
b2v   = sigma_bb*randn(2,1);                    % bias of right reference
z02v  = z2v - b2v;                               % right reference
Jc12m = 1/x02hv(3)^2 * [x02hv(3)*eye(2),-x02hv(1:2)];
Jc22m = 1/z2hv(3)^2  * [z2hv(3) *eye(2),-z2hv(1:2)];
J12m  = Jc12m * JZ2m;
J22m  = Jc22m * Rm * JZ2m;
A002m = J22m / J12m;                             % 2x2 affinity
A2m   = [A002m,b2v;0 0 1];                       % 3x3 affinity
A02m  = [A002m;zeros(1,2)];
Caa2m = diag([sigma_a0^2/scales(2)^4*[1,1,1,1],...
    sigma_b0^2/scales(2)^2*[1,1]]);

%% check constraints
cA2      = [eye(2),x02v;[0 0 1]]'*E_true *[z2v ;1] + ...
           [A002m,z2v  ;[0,0,1]]'*E_true'*[x02v;1];
cp2      = x02hv'*E_true*z2hv;
Ct2      =  (kron([eye(2),x02v;[0 0 1]]',[z2v;1]')...
          +  kron([x02v;1]',[A002m,z2v;[0,0,1]]'));
2*kron([x02v;1]',[z2v;1]');
c2       = Ct2*E_true_t(:);


%% add noise to affinities
dA1 = reshape(rand_gauss(zeros(6,1),Caa1m,1),2,3);
A1m(1:2,:)=A1m(1:2,:) + dA1;
A(:,:,1)=A1m;
dA2 = reshape(rand_gauss(zeros(6,1),Caa2m,1),2,3);
A2m(1:2,:)=A2m(1:2,:) + dA2;

%% Components of AC-pair
% keypoints
u0p = [x01v';x02v'];
v0p = [z01v';z02v'];

% affinities
% Ap(:,:,1)   = A1m;
% Ap(:,:,2)   = A2m;
Ap=[A1m;A2m];

% CovMs
% Caap(:,:,1) = Caa1m;
% Caap(:,:,2) = Caa2m;
Caap = [Caa1m;Caa2m];

% true affinities
% Ap_true(:,:,1)   = A1m;
% Ap_true(:,:,2)   = A2m;
Ap_true=[A1m;A2m];
end

